<?php
error_reporting(E_ALL | E_STRICT);
// first we include phpmorphy library
require_once(dirname(__FILE__) . '/phpmorphy/src/common.php');

class MY_Morphy
{
    var $morphy = '';
    
    function MY_Morphy()
    {
        // set some options
        $opts = array(
            // storage type, follow types supported
            // PHPMORPHY_STORAGE_FILE - use file operations(fread, fseek) for dictionary access, this is very slow...
            // PHPMORPHY_STORAGE_SHM - load dictionary in shared memory(using shmop php extension), this is preferred mode
            // PHPMORPHY_STORAGE_MEM - load dict to memory each time when phpMorphy intialized, this useful when shmop ext. not activated. Speed same as for PHPMORPHY_STORAGE_SHM type
            'storage' => PHPMORPHY_STORAGE_FILE,
            // Extend graminfo for getAllFormsWithGramInfo method call
            'with_gramtab' => false,
            // Enable prediction by suffix
            'predict_by_suffix' => true, 
            // Enable prediction by prefix
            'predict_by_db' => true
        );

        // Path to directory where dictionaries located
        $dir = dirname(__FILE__) . '/phpmorphy/dicts';

        // Create descriptor for dictionary located in $dir directory with russian language
        $dict_bundle = new phpMorphy_FilesBundle($dir, 'rus');

        // Create phpMorphy instance
        try {
            $this->morphy = new phpMorphy($dict_bundle, $opts);
        } catch(phpMorphy_Exception $e) {
            die('Error occured while creating phpMorphy instance: ' . $e->getMessage());
        }
    }
    // получаем существительные слова
    function getKeywords($keywords='',$disallowed=array())
    {
        $_disallowed = array("\n");
        foreach($disallowed as $row)
        {
            $_disallowed[] = $row->name;
            $_disallowed[] = $row->last_name;
        }
        
        $words = explode(' ',iconv('UTF-8','WINDOWS-1251',preg_replace('/\s+/mi',' ',str_replace($_disallowed,'',strip_tags($keywords)))));
        $str_words = '';
        
        foreach($words as $word_one)
        {
            $word_one = mb_strtoupper($word_one,'WINDOWS-1251');
            
            try
            {
                $base_form = $this->morphy->getBaseForm($word_one);
                $all_forms = $this->morphy->getAllForms($word_one);
                $pseudo_root = $this->morphy->getPseudoRoot($word_one);
                $a = $this->morphy->getPartOfSpeech($word_one);   
                
                if(!empty($a) && is_array($a) && array_search(iconv('UTF-8','WINDOWS-1251','С'),$a) !== false)
                    $str_words .= $base_form[0].', ';
                    
            } catch(phpMorphy_Exception $e) {
                die('Error occured while text processing: ' . $e->getMessage());
            }
        }
        return mb_strtolower(iconv('WINDOWS-1251','UTF-8',str_replace(',,',',',rtrim($str_words,', '))),'UTF-8');
    }
}